@@ -9,28 +9,41 @@
  * It is based very heavily on a modified Kerberos v4 kinit, changed to call
  * the Kerberos v5 initialization functions instead.  k5start is not as useful
  * for Kerberos v5 as kstart is for Kerberos v4, since the v5 kinit supports
- * more useful options, but -K and -H are still unique to it.
+ * more useful options, but AFS integration and -K and -H are still unique to
+ * it.
  *
  * Originally written by Robert Morgan and Booker C. Bense.
  * Substantial updates by Russ Allbery <rra@stanford.edu>
  * Copyright 1987, 1988 by the Massachusetts Institute of Technology.
  * Copyright 1995, 1996, 1997, 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007,
- *     2008 Board of Trustees, Leland Stanford Jr. University
+ *     2008, 2009, 2010 Board of Trustees, Leland Stanford Jr. University
  *
  * See LICENSE for licensing terms.
  */
 
 #include <config.h>
 #include <portable/system.h>
-#include <portable/kafs.h>
-#include <portable/time.h>
 
 #include <errno.h>
+#include <krb5.h>
 #include <netdb.h>
 #include <pwd.h>
+#include <signal.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <syslog.h>
+#include <time.h>
 
-#include <util/util.h>
+#include <kafs/kafs.h>
+#include <util/command.h>
+#include <util/concat.h>
+#include <util/macros.h>
+#include <util/messages.h>
+#include <util/messages-krb5.h>
+#include <util/perms.h>
+#include <util/xmalloc.h>
 
 /* The default ticket lifetime in minutes.  Default to 10 hours. */
 #define DEFAULT_LIFETIME (10 * 60)
@@ -52,17 +65,20 @@ struct options {
     char *service;
     krb5_principal ksprinc;
     krb5_ccache ccache;
-    krb5_get_init_creds_opt kopts;
+    krb5_get_init_creds_opt *kopts;
     const char *keytab;
     int happy_ticket;
     int keep_ticket;
-    int quiet;
-    int run_aklog;
+    bool quiet;
+    bool run_aklog;
     const char *aklog;
-    int stdin_passwd;
-    int verbose;
+    bool stdin_passwd;
+    bool verbose;
 };
 
+/* Set when k5start receives SIGALRM. */
+static volatile sig_atomic_t alarm_signaled = 0;
+
 /* The usage message. */
 const char usage_message[] = "\
 Usage: k5start [options] [name [command]]\n\
@@ -84,6 +100,7 @@ Usage: k5start [options] [name [command]]\n\
    -K <interval>        Run as daemon, renew ticket every <interval> minutes\n\
                         (implies -q unless -v is given)\n\
    -k <file>            Use <file> as the ticket cache\n\
+   -L                   Log messages via syslog as well as stderr\n\
    -l <lifetime>        Ticket lifetime in minutes\n\
    -m <mode>            Set ticket cache permissions to <mode> (octal)\n\
    -o <owner>           Set ticket cache owner to <owner>\n\
@@ -101,6 +118,12 @@ If the environment variable AKLOG (or KINIT_PROG for backward compatibility)\n\
 is set to a program (such as aklog) then this program will be executed when\n\
 requested by the -t flag.  Otherwise, %s.\n";
 
+/* Included in the usage message if AFS support is compiled in. */
+const char usage_message_kafs[] = "\n\
+When invoked with -t and a command, k5start will create a new AFS PAG for\n\
+the command before running the AKLOG program to keep its AFS credentials\n\
+isolated from other processes.\n";
+
 
 /*
  * Print out the usage message and then exit with the status given as the
@@ -114,29 +137,42 @@ usage(int status)
             ((PATH_AKLOG[0] == '\0')
              ? "using -t is an error"
              : "the program executed will be\n" PATH_AKLOG));
+#ifdef HAVE_KAFS
+    fprintf((status == 0) ? stdout : stderr, usage_message_kafs);
+#endif
     exit(status);
 }
 
 
+/*
+ * Signal handler for SIGALRM.  Just sets the global sentinel variable.
+ */
+static void
+alarm_handler(int s UNUSED)
+{
+    alarm_signaled = 1;
+}
+
+
 /*
  * Given a context and a principal, get the realm.  This works differently in
  * MIT Kerberos and Heimdal, unfortunately.
  */
-static char *
+static const char *
 get_realm(krb5_context ctx UNUSED, krb5_principal princ)
 {
-#ifdef HAVE_KRB5_REALM
-    krb5_realm *realm;
+#ifdef HAVE_KRB5_PRINCIPAL_GET_REALM
+    const char *realm;
 
-    realm = krb5_princ_realm(ctx, princ);
+    realm = krb5_principal_get_realm(ctx, princ);
     if (realm == NULL)
         die("cannot get local Kerberos realm");
-    return krb5_realm_data(*realm);
+    return realm;
 #else
     krb5_data *data;
 
     data = krb5_princ_realm(ctx, princ);
-    if (data == NULL)
+    if (data == NULL || data->data == NULL)
         die("cannot get local Kerberos realm");
     return data->data;
 #endif
@@ -203,10 +239,10 @@ authenticate(krb5_context ctx, struct options *options)
         if (status != 0)
             warn_krb5(ctx, status, "error unparsing name");
         else {
-            printf("Principal: %s\n", p);
+            notice("authenticating as %s", p);
             free(p);
         }
-        printf("Service principal: %s\n", options->service);
+        notice("getting tickets for %s", options->service);
     }
     if (options->keytab != NULL) {
         status = krb5_kt_resolve(ctx, options->keytab, &keytab);
@@ -215,12 +251,12 @@ authenticate(krb5_context ctx, struct options *options)
                      options->keytab);
         status = krb5_get_init_creds_keytab(ctx, &creds, options->kprinc,
                                             keytab, 0, options->service,
-                                            &options->kopts);
+                                            options->kopts);
     } else if (!options->stdin_passwd) {
         status = krb5_get_init_creds_password(ctx, &creds, options->kprinc,
                                               NULL, krb5_prompter_posix, NULL,
                                               0, options->service,
-                                              &options->kopts);
+                                              options->kopts);
     } else {
         char *p, buffer[BUFSIZ];
 
@@ -235,7 +271,7 @@ authenticate(krb5_context ctx, struct options *options)
         status = krb5_get_init_creds_password(ctx, &creds, options->kprinc,
                                               buffer, NULL, NULL, 0,
                                               options->service,
-                                              &options->kopts);
+                                              options->kopts);
     }
     if (status != 0)
         die_krb5(ctx, status, "error getting credentials");
@@ -315,17 +351,18 @@ main(int argc, char *argv[])
     char **command = NULL;
     char *childfile = NULL;
     char *pidfile = NULL;
-    int background = 0;
-    int nonforwardable = 0;
-    int nonproxiable = 0;
+    bool background = false;
+    bool nonforwardable = false;
+    bool nonproxiable = false;
     int lifetime = DEFAULT_LIFETIME;
     krb5_context ctx;
     krb5_deltat life_secs;
     int status = 0;
     pid_t child = 0;
-    int clean_cache = 0;
-    int search_keytab = 0;
-    static const char optstring[] = "bc:Ff:g:H:hI:i:K:k:l:m:no:Pp:qr:S:stUu:v";
+    bool clean_cache = false;
+    bool search_keytab = false;
+    static const char optstring[]
+        = "bc:Ff:g:H:hI:i:K:k:Ll:m:no:Pp:qr:S:stUu:v";
 
     /* Initialize logging. */
     message_program_name = "k5start";
@@ -334,9 +371,9 @@ main(int argc, char *argv[])
     memset(&options, 0, sizeof(options));
     while ((opt = getopt(argc, argv, optstring)) != EOF)
         switch (opt) {
-        case 'b': background = 1;               break;
+        case 'b': background = true;            break;
         case 'c': childfile = optarg;           break;
-        case 'F': nonforwardable = 1;           break;
+        case 'F': nonforwardable = true;        break;
         case 'g': group = optarg;               break;
         case 'h': usage(0);                     break;
         case 'I': sinst = optarg;               break;
@@ -344,14 +381,14 @@ main(int argc, char *argv[])
         case 'm': mode = optarg;                break;
         case 'n': /* Ignored */                 break;
         case 'o': owner = optarg;               break;
-        case 'P': nonproxiable = 1;             break;
+        case 'P': nonproxiable = true;          break;
         case 'p': pidfile = optarg;             break;
-        case 'q': options.quiet = 1;            break;
+        case 'q': options.quiet = true;         break;
         case 'r': srealm = optarg;              break;
         case 'S': sname = optarg;               break;
-        case 't': options.run_aklog = 1;        break;
-        case 'v': options.verbose = 1;          break;
-        case 'U': search_keytab = 1;            break;
+        case 't': options.run_aklog = true;     break;
+        case 'v': options.verbose = true;       break;
+        case 'U': search_keytab = true;         break;
         case 'u': principal = optarg;           break;
 
         case 'f':
@@ -370,7 +407,19 @@ main(int argc, char *argv[])
                 die("-K interval argument %s out of range", optarg);
             break;
         case 'k':
-            cache = concat("FILE:", optarg, (char *) 0);
+            if (strncmp(optarg, "FILE:", strlen("FILE:")) == 0)
+                cache = xstrdup(optarg);
+            else
+                cache = concat("FILE:", optarg, (char *) 0);
+            break;
+        case 'L':
+            openlog(message_program_name, LOG_PID, LOG_DAEMON);
+            message_handlers_notice(2, message_log_stdout,
+                                    message_log_syslog_notice);
+            message_handlers_warn(2, message_log_stderr,
+                                  message_log_syslog_warning);
+            message_handlers_die(2, message_log_stderr,
+                                 message_log_syslog_err);
             break;
         case 'l':
             code = krb5_string_to_deltat(optarg, &life_secs);
@@ -379,7 +428,7 @@ main(int argc, char *argv[])
             lifetime = life_secs / 60;
             break;
         case 's':
-            options.stdin_passwd = 1;
+            options.stdin_passwd = true;
             if (options.keytab != NULL)
                 die("cannot use both -s and -f flags");
             break;
@@ -475,18 +524,15 @@ main(int argc, char *argv[])
         if (fchmod(fd, 0600) < 0)
             sysdie("cannot chmod ticket cache file");
         cache = tmp;
-        clean_cache = 1;
+        clean_cache = true;
     }
     if (cache == NULL) {
         code = krb5_cc_default(ctx, &options.ccache);
-        if (code != 0)
+        if (code == 0)
             cache = krb5_cc_get_name(ctx, options.ccache);
     } else {
-        char *env;
-
-        if (xasprintf(&env, "KRB5CCNAME=%s", cache) < 0)
-            die("cannot format KRB5CCNAME environment variable");
-        putenv(env);
+        if (setenv("KRB5CCNAME", cache, 1) != 0)
+            die("cannot set KRB5CCNAME environment variable");
         code = krb5_cc_resolve(ctx, cache, &options.ccache);
     }
     if (code != 0)
@@ -498,7 +544,7 @@ main(int argc, char *argv[])
      */
     if (options.keep_ticket > 0 || options.happy_ticket > 0 || background)
         if (!options.verbose)
-            options.quiet = 1;
+            options.quiet = true;
 
     /*
      * The easiest thing for us is if the user just specifies the full
@@ -522,7 +568,10 @@ main(int argc, char *argv[])
     /*
      * Display the identity that we're obtaining Kerberos tickets for.  We do
      * this by unparsing the principal rather than using username and inst
-     * since that way we get the default realm appended by K5.
+     * since that way we get the default realm appended by the Kerberos
+     * libraries.
+     *
+     * We intentionally don't use notice() here to avoid prepending k5start.
      */
     if (!options.quiet) {
         char *p;
@@ -558,22 +607,37 @@ main(int argc, char *argv[])
 
     /* Figure out our ticket lifetime and initialize the options. */
     life_secs = lifetime * 60;
-    krb5_get_init_creds_opt_init(&options.kopts);
+#ifdef HAVE_KRB5_GET_INIT_CREDS_OPT_ALLOC
+    code = krb5_get_init_creds_opt_alloc(ctx, &options.kopts);
+    if (code != 0)
+        die_krb5(ctx, code, "error allocating credential options");
+#else
+    options.kopts = xcalloc(1, sizeof(krb5_get_init_creds_opt));
+    krb5_get_init_creds_opt_init(options.kopts);
+#endif
 #ifdef HAVE_KRB5_GET_INIT_CREDS_OPT_SET_DEFAULT_FLAGS
     krb5_get_init_creds_opt_set_default_flags(ctx, "k5start",
                                               options.kprinc->realm,
-                                              &options.kopts);
+                                              options.kopts);
 #endif
-    krb5_get_init_creds_opt_set_tkt_life(&options.kopts, life_secs);
+    krb5_get_init_creds_opt_set_tkt_life(options.kopts, life_secs);
     if (nonforwardable)
-        krb5_get_init_creds_opt_set_forwardable(&options.kopts, 0);
+        krb5_get_init_creds_opt_set_forwardable(options.kopts, 0);
     if (nonproxiable)
-        krb5_get_init_creds_opt_set_proxiable(&options.kopts, 0);
+        krb5_get_init_creds_opt_set_proxiable(options.kopts, 0);
 
-    /* If we're just checking the service ticket, do that and exit if okay. */
+    /*
+     * If we're just checking the service ticket, do that and exit if okay.
+     * Still re-run aklog if requested, though, since the token may have
+     * expired or we may be initializing a new PAG from an existing ticket
+     * cache.
+     */
     if (options.happy_ticket > 0 && command == NULL)
-        if (!ticket_expired(ctx, &options))
+        if (!ticket_expired(ctx, &options)) {
+            if (options.run_aklog)
+                command_run(options.aklog, options.verbose);
             exit(0);
+        }
 
     /*
      * If built with setpag support and we're running a command, create the
@@ -605,10 +669,11 @@ main(int argc, char *argv[])
         file_permissions(cache, owner, group, mode);
 
     /*
-     * If told to background, background ourselves.  We do this late so that
-     * we can report initial errors.  We have to do this before spawning the
-     * command, though, since we want to background the command as well and
-     * since otherwise we wouldn't be able to wait for the child process.
+     * If told to background, set signal handlers and background ourselves.
+     * We do this late so that we can report initial errors.  We have to do
+     * this before spawning the command, though, since we want to background
+     * the command as well and since otherwise we wouldn't be able to wait for
+     * the child process.
      */
     if (background)
         daemon(0, 0);
@@ -662,7 +727,12 @@ main(int argc, char *argv[])
     /* Loop if we're running as a daemon. */
     if (options.keep_ticket > 0) {
         struct timeval timeout;
+        struct sigaction sa;
 
+        memset(&sa, 0, sizeof(sa));
+        sa.sa_handler = alarm_handler;
+        if (sigaction(SIGALRM, &sa, NULL) < 0)
+            sysdie("cannot set SIGALRM handler");
         while (1) {
             if (command != NULL) {
                 result = command_finish(child, &status);
@@ -674,12 +744,13 @@ main(int argc, char *argv[])
             timeout.tv_sec = options.keep_ticket * 60;
             timeout.tv_usec = 0;
             select(0, NULL, NULL, NULL, &timeout);
-            if (ticket_expired(ctx, &options)) {
+            if (alarm_signaled || ticket_expired(ctx, &options)) {
                 authenticate(ctx, &options);
                 if (options.run_aklog)
                     command_run(options.aklog, options.verbose);
                 if (owner != NULL || group != NULL || mode != NULL)
                     file_permissions(cache, owner, group, mode);
+                alarm_signaled = 0;
             }
         }
     }
