@@ -1,25 +1,14 @@
 /*
  * Test suite for error handling routines.
  *
- * Copyright 2004, 2005, 2006
+ * Written by Russ Allbery <rra@stanford.edu>
+ * Copyright 2009 Board of Trustees, Leland Stanford Jr. University
+ * Copyright (c) 2004, 2005, 2006
  *     by Internet Systems Consortium, Inc. ("ISC")
- * Copyright 1991, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
- *     2003 by The Internet Software Consortium and Rich Salz
+ * Copyright (c) 1991, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+ *     2002, 2003 by The Internet Software Consortium and Rich Salz
  *
- * This code is derived from software contributed to the Internet Software
- * Consortium by Rich Salz.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See LICENSE for licensing terms.
  */
 
 #include <config.h>
@@ -30,8 +19,9 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 
-#include <tests/libtest.h>
-#include <util/util.h>
+#include <tests/tap/basic.h>
+#include <util/concat.h>
+#include <util/messages.h>
 
 #define END (char *) 0
 
@@ -57,10 +47,10 @@ run_test(test_function_t function, char *buf, size_t buflen)
 
     /* Set up the pipe and call the function, collecting its output. */
     if (pipe(fds) == -1)
-        sysdie("can't create pipe");
+        sysbail("can't create pipe");
     child = fork();
     if (child == (pid_t) -1) {
-        sysdie("can't fork");
+        sysbail("can't fork");
     } else if (child == 0) {
         /* In child.  Set up our stdout and stderr. */
         close(fds[0]);
@@ -87,7 +77,7 @@ run_test(test_function_t function, char *buf, size_t buflen)
         } while (status > 0);
         buf[count < 0 ? 0 : count] = '\0';
         if (waitpid(child, &rval, 0) == (pid_t) -1)
-            sysdie("waitpid failed");
+            sysbail("waitpid failed");
     }
     return rval;
 }
@@ -203,42 +193,34 @@ static void test24(void) {
 
 
 /*
- * Given the test number, intended exit status and message, and the function
- * to run, print ok or not ok.
+ * Given the intended exit status and message and the function to run, check a
+ * message test.
  */
 static void
-test_error(int n, int status, const char *output, test_function_t function)
+test_error(int status, const char *output, test_function_t function)
 {
     int real_status;
     char buf[256];
-    int succeeded = 1;
 
     real_status = run_test(function, buf, sizeof(buf));
-    if (!WIFEXITED(real_status) || status != WEXITSTATUS(real_status)) {
-        printf("  unexpected exit status %d\n", real_status);
-        succeeded = 0;
-    }
-    if (strcmp(output, buf)) {
-        printf("  unexpected output: %s", buf);
-        printf("    expected output: %s", output);
-        succeeded = 0;
-    }
-    printf("%sok %d\n", succeeded ? "" : "not ", n);
+    ok(WIFEXITED(real_status), "%d exited", testnum);
+    is_int(status, WEXITSTATUS(real_status), "...with right status");
+    is_string(output, buf, "...and right output");
 }
 
 
 /*
- * Given the test number, intended status, intended message sans the appended
- * strerror output, errno, and the function to run, print ok or not ok.
+ * Given the intended status, intended message sans the appended strerror
+ * output, errno, and the function to run, check the output.
  */
 static void
-test_strerror(int n, int status, const char *output, int error,
+test_strerror(int status, const char *output, int error,
               test_function_t function)
 {
     char *full_output;
 
     full_output = concat(output, ": ", strerror(error), "\n", END);
-    test_error(n, status, full_output, function);
+    test_error(status, full_output, function);
     free(full_output);
 }
 
@@ -251,45 +233,42 @@ main(void)
 {
     char buff[32];
 
-    test_init(24);
+    plan(24 * 3);
 
-    test_error(1, 0, "warning\n", test1);
-    test_error(2, 1, "fatal\n", test2);
-    test_strerror(3, 0, "permissions", EPERM, test3);
-    test_strerror(4, 1, "fatal access", EACCES, test4);
-    test_error(5, 0, "test5: warning\n", test5);
-    test_error(6, 1, "test6: fatal\n", test6);
-    test_strerror(7, 0, "test7: perms 7", EPERM, test7);
-    test_strerror(8, 1, "test8: fatal", EACCES, test8);
-    test_error(9, 10, "fatal\n", test9);
-    test_strerror(10, 10, "fatal perm", EPERM, test10);
-    test_strerror(11, 10, "1st test11: fatal", EPERM, test11);
-    test_error(12, 0, "7 0 warning\n", test12);
-    test_error(13, 1, "5 0 fatal\n", test13);
+    test_error(0, "warning\n", test1);
+    test_error(1, "fatal\n", test2);
+    test_strerror(0, "permissions", EPERM, test3);
+    test_strerror(1, "fatal access", EACCES, test4);
+    test_error(0, "test5: warning\n", test5);
+    test_error(1, "test6: fatal\n", test6);
+    test_strerror(0, "test7: perms 7", EPERM, test7);
+    test_strerror(1, "test8: fatal", EACCES, test8);
+    test_error(10, "fatal\n", test9);
+    test_strerror(10, "fatal perm", EPERM, test10);
+    test_strerror(10, "1st test11: fatal", EPERM, test11);
+    test_error(0, "7 0 warning\n", test12);
+    test_error(1, "5 0 fatal\n", test13);
 
     sprintf(buff, "%d", EPERM);
 
-    test_error(14, 0,
-               concat("7 ", buff, " warning\n7 ", buff, " warning\n", END),
+    test_error(0, concat("7 ", buff, " warning\n7 ", buff, " warning\n", END),
                test14);
-    test_error(15, 10,
-               concat("5 ", buff, " fatal\n5 ", buff, " fatal\n", END),
+    test_error(10, concat("5 ", buff, " fatal\n5 ", buff, " fatal\n", END),
                test15);
-    test_error(16, 0,
-               concat("test16: warning: ", strerror(EPERM), "\n7 ", buff,
-                      " warning\n", END),
+    test_error(0, concat("test16: warning: ", strerror(EPERM), "\n7 ", buff,
+                         " warning\n", END),
                test16);
 
-    test_error(17, 0, "notice\n", test17);
-    test_error(18, 0, "test18: notice\n", test18);
-    test_error(19, 0, "", test19);
-    test_error(20, 0, "3 0 foo\n", test20);
-    test_error(21, 0, "test23: baz\n", test21);
+    test_error(0, "notice\n", test17);
+    test_error(0, "test18: notice\n", test18);
+    test_error(0, "", test19);
+    test_error(0, "3 0 foo\n", test20);
+    test_error(0, "test23: baz\n", test21);
 
     /* Make sure that it's possible to turn off a message type entirely. */ 
-    test_error(22, 1, "", test22);
-    test_error(23, 0, "", test23);
-    test_error(24, 0, "first\nthird\n", test24);
+    test_error(1, "", test22);
+    test_error(0, "", test23);
+    test_error(0, "first\nthird\n", test24);
 
     return 0;
 }
